/*
 * Licensed to STRATIO (C) under one or more contributor license agreements.
 * See the NOTICE file distributed with this work for additional information
 * regarding copyright ownership.  The STRATIO (C) licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package com.stratio.crossdata.common.data;

import com.stratio.crossdata.common.utils.Constants;

/**
 * TABLE name recognized by the parser. The name may contain:
 * <ul>
 * <li>catalog.table</li>
 * <li>table</li>
 * </ul>
 */
public class TableName extends Name {

    private static final long serialVersionUID = -1224192139573160759L;
    private final String name;

    private CatalogName catalogName;

    private String alias = null;

    /**
     * Constructor class.
     * @param catalogName The catalog name.
     * @param tableName The table name.
     */
    public TableName(String catalogName, String tableName) {
        if (catalogName == null || catalogName.isEmpty()) {
            this.catalogName = null;
        } else {
            this.catalogName = new CatalogName(catalogName);
        }
        if(tableName == null || tableName.isEmpty()){
            this.name = null;
        } else {
            this.name = tableName;
        }
    }

    public CatalogName getCatalogName() {
        return catalogName;
    }

    public void setCatalogName(CatalogName catalogName) {
        this.catalogName = catalogName;
    }

    public String getName() {
        return name;
    }

    public String getAlias() {
        return alias;
    }

    public void setAlias(String alias) {
        this.alias = alias;
    }

    /**
     * Return whether the table is virtual or not.
     * @return true if the table is created using a select statement; false if the table is associated to a real data store.
     */
    public boolean isVirtual(){
        return (catalogName != null) ? Constants.VIRTUAL_NAME.equals(catalogName.getName()) : false;
    }

    @Override public boolean isCompletedName() {
        return catalogName != null;
    }

    /**
     * Return the complete name of the table, with te catalog name and the table name.
     * @return A String.
     */
    public String getQualifiedName() {
        String result;
        if (isCompletedName()) {
            result = QualifiedNames.getTableQualifiedName(this.getCatalogName().getName(), getName());
        } else {
            result = QualifiedNames.getTableQualifiedName(UNKNOWN_NAME, getName());
        }
        return result;
    }

    @Override public NameType getType() {
        return NameType.TABLE;
    }

    @Override
    public int hashCode() {
        String code;

        code = getType() + getQualifiedName();

        return code.hashCode();
    }

    @Override
    public boolean equals(Object o) {
        return (this == o) || ((o instanceof TableName) && (this.hashCode() == o.hashCode()));
    }

    @Override public String toString() {
        if(alias == null){
            return this.getQualifiedName();
        }
        StringBuilder sb = new StringBuilder(this.getQualifiedName());
        sb.append(" AS ").append(alias);
        return sb.toString();
    }
}
