/*
 * Licensed to STRATIO (C) under one or more contributor license agreements.
 * See the NOTICE file distributed with this work for additional information
 * regarding copyright ownership.  The STRATIO (C) licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package com.stratio.crossdata.core.statements;

import com.stratio.crossdata.common.data.CatalogName;
import com.stratio.crossdata.common.data.TableName;
import com.stratio.crossdata.core.validator.requirements.ValidationRequirements;
import com.stratio.crossdata.core.validator.requirements.ValidationTypes;

/**
 * Class that models a {@code DROP TABLE} statement from the CROSSDATA language.
 */
public class DropTableStatement extends MetadataStatement {

    /**
     * The name of the target table.
     */
    private TableName tableName;

    /**
     * Whether the table should be dropped only if exists.
     */
    private boolean ifExists;

    /**
     * Whether the drop is an unregister.
     */
    private boolean isExternal=false;

    /**
     * Class constructor.
     *
     * @param tableName The name of the table.
     * @param ifExists  Whether it should be dropped only if exists.
     */
    public DropTableStatement(TableName tableName, boolean ifExists) {
        this.tableName = tableName;
        this.ifExists = ifExists;
    }

    /**
     * Class constructor.
     *
     * @param tableName The name of the table.
     * @param ifExists  Whether it should be dropped only if exists.
     * @param isExternal Whether the operation is a unregister.
     */
    public DropTableStatement(TableName tableName, boolean ifExists, boolean isExternal) {
        this.tableName = tableName;
        this.ifExists = ifExists;
        this.isExternal= isExternal;
    }
    /**
     * Get the name of the table.
     *
     * @return The name.
     */
    public TableName getTableName() {
        return tableName;
    }

    /**
     * Set the name of the table.
     *
     * @param tableName The name of the table.
     */
    public void setTableName(TableName tableName) {
        this.tableName = tableName;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder((isExternal) ? "UNREGISTER TABLE " :"DROP TABLE ");
        if (ifExists) {
            sb.append("IF EXISTS ");
        }
        sb.append(tableName.getQualifiedName());
        return sb.toString();
    }

    /**
     * Get the validation requirements to drop a table.
     * @return A {@link com.stratio.crossdata.core.validator.requirements.ValidationRequirements} .
     */
    public ValidationRequirements getValidationRequirements() {
        return new ValidationRequirements().add(ValidationTypes.MUST_EXIST_CATALOG)
                .add(ValidationTypes.MUST_EXIST_TABLE);
    }

    public boolean isIfExists() {
        return ifExists;
    }

    /**
     * Return if the drop is an unregister operation.
     * @return a boolean.
     */
    public boolean isExternal() {
        return isExternal;
    }

    /**
     * Get the catalogName od the drop table statement.
     * @return A {@link com.stratio.crossdata.common.data.CatalogName} .
     */
    public CatalogName getCatalogName() {
        return tableName.getCatalogName();
    }
}
